@@ -26,6 +26,8 @@
 #include <config.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+
 #include <libbonoboui.h>
 #include <gconf/gconf.h>
 #include <gdk/gdk.h>
@@ -42,7 +44,7 @@
 #include "panel-stock-icons.h"
 #include "xstuff.h"
 
-G_DEFINE_TYPE (PanelAppletFrame, panel_applet_frame, GTK_TYPE_EVENT_BOX);
+G_DEFINE_TYPE (PanelAppletFrame, panel_applet_frame, GTK_TYPE_EVENT_BOX)
 
 #define PANEL_APPLET_FRAME_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET_FRAME, PanelAppletFramePrivate))
 
@@ -96,14 +98,9 @@ static GSList *no_reload_applets = NULL;
 static void panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
 					   PanelAppletFrame *frame);
 
-static char *panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
-						   PanelWidget      *panel,
-						   const char       *iid,
-						   const char       *id);
-
-static void panel_applet_frame_activated (Bonobo_Unknown     object,
-					  CORBA_Environment *ev,
-					  gpointer          *data);
+static void panel_applet_frame_activated (CORBA_Object object,
+					  const char   *error_reason,
+					  gpointer     data);
 
 void
 panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
@@ -335,7 +332,6 @@ panel_applet_frame_load (const gchar *iid,
 {
 	PanelAppletFrame           *frame;
 	CORBA_Environment           ev;
-	char                       *moniker;
 	PanelAppletFrameActivating *frame_act;
 
 	g_return_if_fail (iid != NULL);
@@ -360,15 +356,13 @@ panel_applet_frame_load (const gchar *iid,
 	frame_act->exactpos = exactpos;
 	frame_act->id       = g_strdup (id);
 
-	moniker = panel_applet_frame_construct_moniker (frame, panel, iid, id);
-
 	CORBA_exception_init (&ev);
 
-	bonobo_get_object_async (moniker, "IDL:Bonobo/Control:1.0", &ev,
-				 (BonoboMonikerAsyncFn) panel_applet_frame_activated,
-				 frame_act);
+	bonobo_activation_activate_from_id_async (frame->priv->iid, 0,
+						  (BonoboActivationCallback) panel_applet_frame_activated,
+						  frame_act, &ev);
 
-	g_free (moniker);
+	CORBA_exception_free (&ev);
 }
 
 void
@@ -550,7 +544,7 @@ panel_applet_frame_finalize (GObject *object)
 		 * like we can receive some events when unref'ing them */
 		ORBit_small_unlisten_for_broken (frame->priv->control,
 						 G_CALLBACK (panel_applet_frame_cnx_broken));
-		CORBA_Object_release (frame->priv->control, NULL);
+		bonobo_object_release_unref (frame->priv->control, NULL);
 		frame->priv->control = CORBA_OBJECT_NIL;
 	}
 
@@ -1061,7 +1055,7 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 	dialog = gtk_message_dialog_new (NULL, 0,
 					 locked_down ? GTK_MESSAGE_INFO : GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_NONE,
-					 problem_txt);
+					 "%s", problem_txt);
 	g_free (problem_txt);
 
 	if (locked_down) {
@@ -1209,10 +1203,9 @@ panel_applet_frame_get_size_string (PanelAppletFrame *frame,
 }
 
 static char *
-panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
-				      PanelWidget      *panel,
-				      const char       *iid,
-				      const char       *id)
+panel_applet_frame_construct_item (PanelAppletFrame *frame,
+				   PanelWidget      *panel,
+				   const char       *id)
 {
 	char *retval;
 	char *bg_str;
@@ -1227,9 +1220,9 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 	locked_down = panel_lockdown_get_locked_down ();
 
 	retval = g_strdup_printf (
-			"%s!prefs_key=/apps/panel/applets/%s/prefs;"
+			"prefs_key=/apps/panel/applets/%s/prefs;"
 			"background=%s;orient=%s;size=%s;locked_down=%s",
-			iid, id, bg_str,
+			id, bg_str,
 			panel_applet_frame_get_orient_string (frame, panel),
 			panel_applet_frame_get_size_string (frame, panel),
 			locked_down ? "true" : "false");
@@ -1254,60 +1247,90 @@ panel_applet_frame_event_listener (BonoboListener    *listener,
 }
 
 static void
-panel_applet_frame_activated (Bonobo_Unknown     object,
-			      CORBA_Environment *ev,
-			      gpointer          *data)
+panel_applet_frame_activated (CORBA_Object  object,
+			      const char   *error_reason,
+			      gpointer      data)
 {
 	PanelAppletFrameActivating *frame_act;
 	PanelAppletFrame   *frame;
 	GtkWidget          *widget;
 	BonoboControlFrame *control_frame;
 	Bonobo_Control      control;
+	Bonobo_ItemContainer container;
 	CORBA_Environment   corba_ev;
 	AppletInfo         *info;
 	char               *error;
+	char               *item_name;
 
+	widget = NULL;
 	frame_act = (PanelAppletFrameActivating *) data;
 	frame = frame_act->frame;
 
 	/* according to the source of bonobo control == NULL && no
 	   exception can happen, so handle it */
-	if (BONOBO_EX (ev) || object == CORBA_OBJECT_NIL) {
-		error = bonobo_exception_get_text (ev);
+	if (error_reason != NULL || object == CORBA_OBJECT_NIL) {
 		g_warning (G_STRLOC ": failed to load applet %s:\n%s",
-			   frame->priv->iid, error);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		g_free (frame_act->id);
-		g_free (frame_act);
-		g_free (error);
-		return;
+			   frame->priv->iid, error_reason);
+		goto error_out;
 	}
 
-	frame->priv->control = CORBA_Object_duplicate (object, NULL);
+	CORBA_exception_init (&corba_ev);
 
-	widget = bonobo_widget_new_control_from_objref (object,
+	item_name = panel_applet_frame_construct_item (frame,
+						       frame->priv->panel,
+						       frame_act->id);
+
+	frame->priv->control = CORBA_OBJECT_NIL;
+	container = Bonobo_Unknown_queryInterface (object,
+						   "IDL:Bonobo/ItemContainer:1.0",
+						   &corba_ev);
+	if (!BONOBO_EX (&corba_ev) && container != CORBA_OBJECT_NIL) {
+		Bonobo_Unknown containee;
+
+		containee = Bonobo_ItemContainer_getObjectByName (container,
+								  item_name,
+								  TRUE,
+								  &corba_ev);
+		bonobo_object_release_unref (container, NULL);
+
+		if (!BONOBO_EX (&corba_ev) && containee != CORBA_OBJECT_NIL) {
+			frame->priv->control =
+				Bonobo_Unknown_queryInterface (containee,
+							       "IDL:Bonobo/Control:1.0",
+							       &corba_ev);
+
+			bonobo_object_release_unref (containee, NULL);
+		}
+	}
+	g_free (item_name);
+
+	if (frame->priv->control == CORBA_OBJECT_NIL) {
+		error = bonobo_exception_get_text (&corba_ev);
+		g_warning (G_STRLOC ": failed to get Bonobo/Control interface on applet %s:\n%s",
+                           frame->priv->iid, error);
+		CORBA_exception_free (&corba_ev);
+		bonobo_object_release_unref (object, NULL);
+		g_free (error);
+		goto error_out;
+	}
+
+	widget = bonobo_widget_new_control_from_objref (frame->priv->control,
 							CORBA_OBJECT_NIL);
 
+	CORBA_exception_free (&corba_ev);
 	bonobo_object_release_unref (object, NULL);
 
 	if (!widget) {
 		g_warning (G_STRLOC ": failed to load applet %s",
 			   frame->priv->iid);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		g_free (frame_act->id);
-		g_free (frame_act);
-		return;
+		goto error_out;
 	}
 
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
 	if (control_frame == NULL) {
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(cannot get control frame)", frame->priv->iid);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		gtk_object_sink (GTK_OBJECT (widget));
-		g_free (frame_act->id);
-		g_free (frame_act);
-		return;
+		goto error_out;
 	}
 
 	frame->priv->property_bag = 
@@ -1319,12 +1342,8 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(cannot get property bag):\n%s",
 			   frame->priv->iid, error);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		gtk_object_sink (GTK_OBJECT (widget));
-		g_free (frame_act->id);
-		g_free (frame_act);
 		g_free (error);
-		return;
+		goto error_out;
 	}
 
 	bonobo_event_source_client_add_listener (frame->priv->property_bag,
@@ -1342,12 +1361,8 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(cannot get popup component):\n%s",
 			   frame->priv->iid, error);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		gtk_object_sink (GTK_OBJECT (widget));
-		g_free (frame_act->id);
-		g_free (frame_act);
 		g_free (error);
-		return;
+		goto error_out;
 	}
 
 	bonobo_ui_util_set_ui (frame->priv->ui_component, DATADIR,
@@ -1366,11 +1381,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(cannot get control)", frame->priv->iid);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		gtk_object_sink (GTK_OBJECT (widget));
-		g_free (frame_act->id);
-		g_free (frame_act);
-		return;
+		goto error_out;
 	}
 
 	frame->priv->applet_shell = panel_applet_frame_get_applet_shell (control);
@@ -1378,11 +1389,7 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 		CORBA_exception_free (&corba_ev);
 		g_warning (G_STRLOC ": failed to load applet %s "
 			   "(cannot get applet shell)", frame->priv->iid);
-		panel_applet_frame_loading_failed (frame, frame_act->id);
-		gtk_object_sink (GTK_OBJECT (widget));
-		g_free (frame_act->id);
-		g_free (frame_act);
-		return;
+		goto error_out;
 	}
 
 	CORBA_exception_free (&corba_ev);
@@ -1411,6 +1418,17 @@ panel_applet_frame_activated (Bonobo_Unknown     object,
 	panel_lockdown_notify_add (G_CALLBACK (panel_applet_frame_sync_menu_state),
 				   frame);
 
+	panel_applet_stop_loading (frame_act->id);
+	g_free (frame_act->id);
+	g_free (frame_act);
+
+	return;
+
+error_out:
+	panel_applet_frame_loading_failed (frame, frame_act->id);
+	if (widget)
+		g_object_unref (widget);
+	panel_applet_stop_loading (frame_act->id);
 	g_free (frame_act->id);
 	g_free (frame_act);
 }
