@@ -35,6 +35,7 @@
 #include <gdk/gdkkeysyms.h>
 
 #include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-show.h>
 
 #include "nothing.h"
 #include "panel-util.h"
@@ -53,7 +54,7 @@ panel_context_menu_show_help (GtkWidget *w,
 			      gpointer data)
 {
 	panel_show_help (gtk_widget_get_screen (w),
-			 "user-guide.xml", "gospanel-1");
+			 "user-guide", "gospanel-1", NULL);
 }
 
 static gboolean
@@ -89,7 +90,7 @@ panel_context_menu_show_about_dialog (GtkWidget *menuitem)
 		"George Lebl (jirka@5z.com)",
 		"Glynn Foster (glynn.foster@sun.com)",
 		"Ian Main (imain@gtk.org)",
-		"Ian McKellar <yakk@yakk.net>",
+		"Ian McKellar (yakk@yakk.net)",
 		"Jacob Berkman (jberkman@andrew.cmu.edu)",
 		"Mark McLoughlin (mark@skynet.ie)",
 		"Martin Baulig (baulig@suse.de)",
