@@ -27,10 +27,15 @@
 #include "panel-properties-dialog.h"
 
 #include <string.h>
-#include <glade/glade-xml.h>
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-icon-entry.h>
 
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-gtk.h>
+#include <libpanel-util/panel-show.h>
+
+#include "nothing.h"
 #include "panel-profile.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
@@ -76,6 +81,9 @@ typedef struct {
 
 	/* The theme directory of the icon, see bug #119209 */
 	char          *icon_theme_dir;
+
+	/* FIXME: This is a workaround for GTK+ bug #327243 */
+	int            selection_emitted;
 } PanelPropertiesDialog;
 
 static GQuark panel_properties_dialog_quark = 0;
@@ -117,10 +125,10 @@ typedef struct {
 } OrientationComboItem;
 
 static OrientationComboItem orientation_items [] = {
-	{ N_("Orientation|Top"),    PANEL_ORIENTATION_TOP    },
-	{ N_("Orientation|Bottom"), PANEL_ORIENTATION_BOTTOM },
-	{ N_("Orientation|Left"),   PANEL_ORIENTATION_LEFT   },
-	{ N_("Orientation|Right"),  PANEL_ORIENTATION_RIGHT  }
+	{ NC_("Orientation", "Top"),    PANEL_ORIENTATION_TOP    },
+	{ NC_("Orientation", "Bottom"), PANEL_ORIENTATION_BOTTOM },
+	{ NC_("Orientation", "Left"),   PANEL_ORIENTATION_LEFT   },
+	{ NC_("Orientation", "Right"),  PANEL_ORIENTATION_RIGHT  }
 };
 
 static void
@@ -147,7 +155,7 @@ panel_properties_dialog_orientation_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_orientation_combo (PanelPropertiesDialog *dialog,
-						 GladeXML              *gui)
+						 GtkBuilder            *gui)
 {
 	PanelOrientation  orientation;
 	GtkListStore     *model;
@@ -155,9 +163,9 @@ panel_properties_dialog_setup_orientation_combo (PanelPropertiesDialog *dialog,
 	GtkCellRenderer  *renderer;
 	int               i;
 
-	dialog->orientation_combo = glade_xml_get_widget (gui, "orientation_combo");
+	dialog->orientation_combo = PANEL_GTK_BUILDER_GET (gui, "orientation_combo");
 	g_return_if_fail (dialog->orientation_combo != NULL);
-	dialog->orientation_label = glade_xml_get_widget (gui, "orientation_label");
+	dialog->orientation_label = PANEL_GTK_BUILDER_GET (gui, "orientation_label");
 	g_return_if_fail (dialog->orientation_label != NULL);
 
 	orientation = panel_profile_get_toplevel_orientation (dialog->toplevel);
@@ -172,7 +180,7 @@ panel_properties_dialog_setup_orientation_combo (PanelPropertiesDialog *dialog,
 	for (i = 0; i < G_N_ELEMENTS (orientation_items); i++) {
 		gtk_list_store_append (model, &iter);
 		gtk_list_store_set (model, &iter,
-				    COLUMN_TEXT, Q_(orientation_items [i].name),
+				    COLUMN_TEXT, g_dpgettext2 (NULL, "Orientation", orientation_items [i].name),
 				    COLUMN_ITEM, &(orientation_items [i]),
 				    -1);
 		if (orientation == orientation_items [i].orientation)
@@ -205,23 +213,22 @@ panel_properties_dialog_size_changed (PanelPropertiesDialog *dialog,
 					 gtk_spin_button_get_value_as_int (spin_button));
 }
 
-#define TOPLEVEL_MAX_SIZE 120
 static void
 panel_properties_dialog_setup_size_spin (PanelPropertiesDialog *dialog,
-					 GladeXML              *gui)
+					 GtkBuilder            *gui)
 {
-	dialog->size_widgets = glade_xml_get_widget (gui, "size_widgets");
+	dialog->size_widgets = PANEL_GTK_BUILDER_GET (gui, "size_widgets");
 	g_return_if_fail (dialog->size_widgets != NULL);
-	dialog->size_spin = glade_xml_get_widget (gui, "size_spin");
+	dialog->size_spin = PANEL_GTK_BUILDER_GET (gui, "size_spin");
 	g_return_if_fail (dialog->size_spin != NULL);
-	dialog->size_label = glade_xml_get_widget (gui, "size_label");
+	dialog->size_label = PANEL_GTK_BUILDER_GET (gui, "size_label");
 	g_return_if_fail (dialog->size_label != NULL);
-	dialog->size_label_pixels = glade_xml_get_widget (gui, "size_label_pixels");
+	dialog->size_label_pixels = PANEL_GTK_BUILDER_GET (gui, "size_label_pixels");
 	g_return_if_fail (dialog->size_label_pixels != NULL);
 
 	gtk_spin_button_set_range (GTK_SPIN_BUTTON (dialog->size_spin),
 				   panel_toplevel_get_minimum_size (dialog->toplevel),
-				   TOPLEVEL_MAX_SIZE);
+				   panel_toplevel_get_maximum_size (dialog->toplevel));
 
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->size_spin),
 				   panel_profile_get_toplevel_size (dialog->toplevel));
@@ -268,17 +275,20 @@ panel_properties_dialog_icon_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
-					  GladeXML              *gui)
+					  GtkBuilder            *gui)
 {
 	char *custom_icon;
 
-	dialog->icon_align = glade_xml_get_widget (gui, "icon_align");
+	dialog->icon_align = PANEL_GTK_BUILDER_GET (gui, "icon_align");
 	g_return_if_fail (dialog->icon_align != NULL);
 
-	dialog->icon_entry = glade_xml_get_widget (gui, "icon_entry");
-	g_return_if_fail (dialog->icon_entry != NULL);
+	dialog->icon_entry = gnome_icon_entry_new ("panel-drawer-icon",
+						   _("Browse icons"));
+	gtk_widget_show (dialog->icon_entry);
+	gtk_container_add (GTK_CONTAINER (dialog->icon_align),
+			   dialog->icon_entry);
 
-	dialog->icon_label = glade_xml_get_widget (gui, "icon_label");
+	dialog->icon_label = PANEL_GTK_BUILDER_GET (gui, "icon_label");
 	g_return_if_fail (dialog->icon_label != NULL);
 
 	dialog->icon_theme_dir = NULL;
@@ -330,9 +340,9 @@ panel_properties_dialog_setup_icon_entry (PanelPropertiesDialog *dialog,
 	}                                                                                         \
 	static void                                                                               \
 	panel_properties_dialog_setup_##n (PanelPropertiesDialog *dialog,                         \
-					   GladeXML              *gui)                            \
+					   GtkBuilder            *gui)                            \
 	{                                                                                         \
-		dialog->n = glade_xml_get_widget (gui, wid);                                      \
+		dialog->n = PANEL_GTK_BUILDER_GET (gui, wid);                                      \
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->n),                      \
 					      panel_profile_get_toplevel_##p (dialog->toplevel)); \
 		g_signal_connect_swapped (dialog->n, "toggled",                                   \
@@ -362,13 +372,13 @@ panel_properties_dialog_color_changed (PanelPropertiesDialog *dialog,
 
 static void
 panel_properties_dialog_setup_color_button (PanelPropertiesDialog *dialog,
-					    GladeXML              *gui)
+					    GtkBuilder            *gui)
 {
 	PanelColor color;
 
-	dialog->color_button = glade_xml_get_widget (gui, "color_button");
+	dialog->color_button = PANEL_GTK_BUILDER_GET (gui, "color_button");
 	g_return_if_fail (dialog->color_button != NULL);
-	dialog->color_label = glade_xml_get_widget (gui, "color_label");
+	dialog->color_label = PANEL_GTK_BUILDER_GET (gui, "color_label");
 	g_return_if_fail (dialog->color_label != NULL);
 
 	panel_profile_get_background_color (dialog->toplevel, &color);
@@ -394,6 +404,12 @@ panel_properties_dialog_image_changed (PanelPropertiesDialog *dialog)
 
 	image = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog->image_chooser));
 
+	/* FIXME: This is an ugly workaround for GTK+ bug #327243.
+	 * FIXME: Note that GTK+ 2.12 and file-set signal might help. */
+	if (!dialog->selection_emitted < 2 && !image) {
+		dialog->selection_emitted++;
+		return;
+	}
 	panel_profile_set_background_image (dialog->toplevel, image);
 
 	g_free (image);
@@ -428,19 +444,16 @@ panel_properties_dialog_chooser_preview_update (GtkFileChooser *file_chooser,
 
 static void
 panel_properties_dialog_setup_image_chooser (PanelPropertiesDialog *dialog,
-					     GladeXML              *gui)
+					     GtkBuilder            *gui)
 {
 	GtkFileFilter *filter;
 	GtkWidget     *chooser_preview;
 	char          *image;
 
-	dialog->image_chooser = glade_xml_get_widget (gui, "image_chooser");
+	dialog->image_chooser = PANEL_GTK_BUILDER_GET (gui, "image_chooser");
 
 	filter = gtk_file_filter_new ();
-	gtk_file_filter_set_name (filter, _("Images"));
 	gtk_file_filter_add_pixbuf_formats (filter);
-	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog->image_chooser),
-				     filter);
 	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog->image_chooser),
 				     filter);
 
@@ -453,7 +466,7 @@ panel_properties_dialog_setup_image_chooser (PanelPropertiesDialog *dialog,
 
 	image = panel_profile_get_background_image (dialog->toplevel);
 
-	if (string_empty (image))
+	if (PANEL_GLIB_STR_EMPTY (image))
 		gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER (dialog->image_chooser));
 	else
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog->image_chooser),
@@ -462,6 +475,7 @@ panel_properties_dialog_setup_image_chooser (PanelPropertiesDialog *dialog,
 	if (image)
 		g_free (image);
 
+	dialog->selection_emitted = 0;
 	g_signal_connect_swapped (dialog->image_chooser, "selection-changed",
 				  G_CALLBACK (panel_properties_dialog_image_changed),
 				  dialog);
@@ -492,16 +506,16 @@ panel_properties_dialog_opacity_changed (PanelPropertiesDialog *dialog)
 
 static void
 panel_properties_dialog_setup_opacity_scale (PanelPropertiesDialog *dialog,
-					     GladeXML              *gui)
+					     GtkBuilder            *gui)
 {
 	guint16 opacity;
 	gdouble percentage;
 
-	dialog->opacity_scale = glade_xml_get_widget (gui, "opacity_scale");
+	dialog->opacity_scale = PANEL_GTK_BUILDER_GET (gui, "opacity_scale");
 	g_return_if_fail (dialog->opacity_scale != NULL);
-	dialog->opacity_label = glade_xml_get_widget (gui, "opacity_label");
+	dialog->opacity_label = PANEL_GTK_BUILDER_GET (gui, "opacity_label");
 	g_return_if_fail (dialog->opacity_label != NULL);
-	dialog->opacity_legend = glade_xml_get_widget (gui, "opacity_legend");
+	dialog->opacity_legend = PANEL_GTK_BUILDER_GET (gui, "opacity_legend");
 	g_return_if_fail (dialog->opacity_legend != NULL);
 
 	opacity = panel_profile_get_background_opacity (dialog->toplevel);
@@ -557,16 +571,16 @@ panel_properties_dialog_background_toggled (PanelPropertiesDialog *dialog,
 				
 static void
 panel_properties_dialog_setup_background_radios (PanelPropertiesDialog *dialog,
-						 GladeXML              *gui)
+						 GtkBuilder            *gui)
 {
 	PanelBackgroundType  background_type;
 	GtkWidget           *active_radio;
 
-	dialog->default_radio     = glade_xml_get_widget (gui, "default_radio");
-	dialog->color_radio       = glade_xml_get_widget (gui, "color_radio");
-	dialog->image_radio       = glade_xml_get_widget (gui, "image_radio");
-	dialog->color_widgets     = glade_xml_get_widget (gui, "color_widgets");
-	dialog->image_widgets     = glade_xml_get_widget (gui, "image_widgets");
+	dialog->default_radio     = PANEL_GTK_BUILDER_GET (gui, "default_radio");
+	dialog->color_radio       = PANEL_GTK_BUILDER_GET (gui, "color_radio");
+	dialog->image_radio       = PANEL_GTK_BUILDER_GET (gui, "image_radio");
+	dialog->color_widgets     = PANEL_GTK_BUILDER_GET (gui, "color_widgets");
+	dialog->image_widgets     = PANEL_GTK_BUILDER_GET (gui, "image_widgets");
 
 	background_type = panel_profile_get_background_type (dialog->toplevel);
 	switch (background_type) {
@@ -635,7 +649,7 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 			help_id = "gospanel-28";
 		}
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (properties_dialog)),
-				 "user-guide.xml", help_id);
+				 "user-guide", help_id, NULL);
 		break;
 	default:
 		break;
@@ -659,6 +673,9 @@ panel_properties_dialog_update_orientation (PanelPropertiesDialog *dialog,
 	GtkTreeModel         *model;
 	GtkTreeIter           iter;
 	OrientationComboItem *item;
+	int                   max_size;
+	int                   spin_size;
+	int                   profile_size;
 
 	if (!value || value->type != GCONF_VALUE_STRING)
 		return;
@@ -666,6 +683,24 @@ panel_properties_dialog_update_orientation (PanelPropertiesDialog *dialog,
 	if (!panel_profile_map_orientation_string (gconf_value_get_string (value), &orientation))
 		return;
 
+	/* change the maximum size of the panel */
+	//TODO: we should also do this when the monitor size changes
+	max_size = panel_toplevel_get_maximum_size (dialog->toplevel);
+	spin_size = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (dialog->size_spin));
+	profile_size = panel_profile_get_toplevel_size (dialog->toplevel);
+
+	gtk_spin_button_set_range (GTK_SPIN_BUTTON (dialog->size_spin),
+				   panel_toplevel_get_minimum_size (dialog->toplevel),
+				   max_size);
+
+	if (spin_size > max_size)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->size_spin),
+					   max_size);
+	else if (spin_size != profile_size)
+		gtk_spin_button_set_value (GTK_SPIN_BUTTON (dialog->size_spin),
+					   MIN (profile_size, max_size));
+
+	/* update the orientation combo box */
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->orientation_combo));
 
 	if (!gtk_tree_model_get_iter_first (model, &iter))
@@ -810,9 +845,10 @@ panel_properties_dialog_update_background_image (PanelPropertiesDialog *dialog,
 	text = gconf_value_get_string (value);
 	old_text = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog->image_chooser));
 
-	if (string_empty (text) && old_text)
+	if (PANEL_GLIB_STR_EMPTY (text) && old_text)
 		gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER (dialog->image_chooser));
-	else if (!string_empty (text) && (!old_text || strcmp (text, old_text)))
+	else if (!PANEL_GLIB_STR_EMPTY (text) &&
+		 (!old_text || strcmp (text, old_text)))
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (dialog->image_chooser),
 					       text);
 
@@ -912,6 +948,8 @@ panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
 	if (!attached)
 		panel_properties_dialog_remove_icon_entry (dialog);
 	else {
+		gtk_window_set_title (GTK_WINDOW (dialog->properties_dialog),
+				      _("Drawer Properties"));
 		panel_properties_dialog_remove_toggles (dialog);
 		panel_properties_dialog_remove_orientation_combo (dialog);
 	}
@@ -919,7 +957,7 @@ panel_properties_dialog_update_for_attached (PanelPropertiesDialog *dialog,
 
 static PanelPropertiesDialog *
 panel_properties_dialog_new (PanelToplevel *toplevel,
-			     GladeXML      *gui)
+			     GtkBuilder    *gui)
 {
 	PanelPropertiesDialog *dialog;
 
@@ -932,7 +970,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 
 	dialog->toplevel = toplevel;
 
-	dialog->properties_dialog = glade_xml_get_widget (gui, "panel_properties_dialog");
+	dialog->properties_dialog = PANEL_GTK_BUILDER_GET (gui, "panel_properties_dialog");
 	g_signal_connect_swapped (dialog->properties_dialog, "response",
 				  G_CALLBACK (panel_properties_dialog_response), dialog);
 	g_signal_connect_swapped (dialog->properties_dialog, "destroy",
@@ -941,11 +979,11 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	gtk_window_set_screen (GTK_WINDOW (dialog->properties_dialog),
 			       gtk_window_get_screen (GTK_WINDOW (toplevel)));
 
-	dialog->writability_warn_general = glade_xml_get_widget (gui, "writability_warn_general");
-	dialog->writability_warn_background = glade_xml_get_widget (gui, "writability_warn_background");
+	dialog->writability_warn_general = PANEL_GTK_BUILDER_GET (gui, "writability_warn_general");
+	dialog->writability_warn_background = PANEL_GTK_BUILDER_GET (gui, "writability_warn_background");
 
-	dialog->general_vbox  = glade_xml_get_widget (gui, "general_vbox");
-	dialog->general_table = glade_xml_get_widget (gui, "general_table");
+	dialog->general_vbox  = PANEL_GTK_BUILDER_GET (gui, "general_vbox");
+	dialog->general_table = PANEL_GTK_BUILDER_GET (gui, "general_table");
 
 	panel_properties_dialog_setup_orientation_combo  (dialog, gui);
 	panel_properties_dialog_setup_size_spin          (dialog, gui);
@@ -987,6 +1025,10 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	panel_widget_register_open_dialog (panel_toplevel_get_panel_widget (dialog->toplevel),
 					   dialog->properties_dialog);
 
+	g_signal_connect (dialog->properties_dialog, "event",
+			  G_CALLBACK (config_event),
+			  PANEL_GTK_BUILDER_GET (gui, "notebook"));
+
 	gtk_widget_show (dialog->properties_dialog);
 
 	return dialog;
@@ -996,7 +1038,8 @@ void
 panel_properties_dialog_present (PanelToplevel *toplevel)
 {
 	PanelPropertiesDialog *dialog;
-	GladeXML              *gui;
+	GtkBuilder            *gui;
+	GError                *error;
 
 	if (!panel_properties_dialog_quark)
 		panel_properties_dialog_quark =
@@ -1010,19 +1053,29 @@ panel_properties_dialog_present (PanelToplevel *toplevel)
 		return;
 	}
 
-	gui = glade_xml_new (GLADEDIR "/panel-properties-dialog.glade",
-			     "panel_properties_dialog",
-			     NULL);
-	if (gui == NULL) {
+	gui = gtk_builder_new ();
+	gtk_builder_set_translation_domain (gui, GETTEXT_PACKAGE);
+
+	error = NULL;
+	gtk_builder_add_from_file (gui,
+				   BUILDERDIR "/panel-properties-dialog.ui",
+				   &error);
+
+        if (error) {
 		char *secondary;
-		secondary = g_strdup_printf (_("Unable to load file '%s'."),
-					     GLADEDIR"/panel-properties-dialog.glade");
-		panel_error_dialog (NULL,
+
+		secondary = g_strdup_printf (_("Unable to load file '%s': %s."),
+					     BUILDERDIR"/panel-properties-dialog.ui",
+					     error->message);
+		panel_error_dialog (GTK_WINDOW (toplevel),
 				    gtk_window_get_screen (GTK_WINDOW (toplevel)),
 				    "cannot_display_properties_dialog", TRUE,
 				    _("Could not display properties dialog"),
 				    secondary);
 		g_free (secondary);
+		g_error_free (error);
+		g_object_unref (gui);
+
 		return;
 	}
 
