@@ -25,13 +25,16 @@
 #ifndef __PANEL_STOCK_ICONS_H__
 #define __PANEL_STOCK_ICONS_H__
 
-#include <glib/gmacros.h>
-#include <gtk/gtkenums.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-/* themeable size - "panel-menu" */
-#define PANEL_DEFAULT_MENU_ICON_SIZE 	24
+/* themeable size - "panel-menu" -- This is used for the icons in the menus */
+#define PANEL_DEFAULT_MENU_ICON_SIZE 		24
+/* themeable size - "panel-foobar" -- This is only used for the icon of the
+ * Applications item in the menu bar */
+#define PANEL_DEFAULT_MENU_BAR_ICON_SIZE 	24
 
 /* stock icons */
 #define PANEL_STOCK_FORCE_QUIT          "gnome-panel-force-quit"
